#    zwm -- zadka's window manager
#    Copyright (C) 2002 Moshe Zadka <mo@moshez.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys, os, time

from Xlib import X, Xatom

from plwm import wmanager, focus, keys, \
     deltamove, outline, \
     border, color, font, views, \
     modewindow, modestatus, \
     mw_clock

from plwm.cycle import CycleKeys
from plwm.moveresize import MoveResizeKeys
from plwm.cfilter import *

delta = deltamove.DeltaMove()

class ZWMClient(wmanager.Client,
               outline.XorOutlineClient,
               border.BorderClient,
               modestatus.ModeFocusedTitleClient):
    
    no_border_clients = none
    

class ZWMScreen(wmanager.Screen,
               color.Color,
               modewindow.ModeWindowScreen,
               modestatus.ModeStatus,
               modestatus.ModeMoveResize,
               views.XMW_ViewHandler):

    view_always_visible_clients = none

class MouseHandler:

    def __init__(self, wmanager):
        self.wm = wmanager
        wmanager.dispatch.add_handler(X.ButtonPress, self)
        wmanager.dispatch.add_handler(X.ButtonRelease, self)

    def __call__(self, evt):
        name = 'handle_B%d' % evt.detail
        if (evt.type == X.ButtonRelease):
            name = name + '_Release' 
        else:
            name = name + '_Press' 
        meth = getattr(self, name, None)
        if meth:
            meth(evt)


class ZWMMouseHandler(MouseHandler):

    def handle_B1_Release(self, evt):
        self.wm.system('xterm -geometry 80x25+%d+%d &' % 
                       (evt.event_x, evt.event_y))

    def handle_B2_Release(self, evt):
        self.wm.quit()

    def handle_B3_Release(self, evt):
        self.wm.system('/usr/bin/zwmenu')


class PropertyHandler:

    def __init__(self, wmanager):
        self.wm = wmanager
        self.cache = {}
        wmanager.default_screen.dispatch.add_handler(X.PropertyNotify, self)

    def get_atom_name(self, atom):
        if self.cache.has_key(atom):
            return self.cache[atom]
        ret = self.cache[atom] = self.wm.display.get_atom_name(atom)
        return ret

    def __call__(self, evt):
        name = 'handle_'+self.get_atom_name(evt.atom)
        meth = getattr(self, name, None)
        if meth:
            return meth(evt)

    def handle_ZWM_QUIT(self, evt):
        self.wm.quit()

    def handle_ZWM_EXEC(self, evt):
        property = self.wm.display.screen().root.get_full_property(evt.atom,
                                                             Xatom.STRING)
        program = property.value
	os.execl(program, program)



class WMConfig:
    def __wm_init__(self):
        BasicKeys(self, self.dispatch)
        ZWMMouseHandler(self)
        PropertyHandler(self)


class ZWM(wmanager.WindowManager,
           focus.SloppyFocus,
           focus.MoveFocus,
           font.Font,
           mw_clock.ModeWindowClock,
           WMConfig):

    client_class = ZWMClient
    screen_class = ZWMScreen
    

class BasicKeys(keys.KeyHandler):

    def S_Pause(self, evt):
        wmanager.debug('keys', 'dropping keygrabs temporarily')

        # First release all our grabs.  They will be reinstalled
        # by BypassHandler when it exits
        self._ungrab()
        BypassHandler(self)


    # Window control

    def M1_u(self, evt):
        if self.wm.focus_client:
            self.wm.focus_client.configure(stack_mode = X.Opposite)
    
    def M1_i(self, evt):
        if self.wm.focus_client:
            self.wm.focus_client.iconify()

    def M1_d(self, evt):
        CycleUMKeys(self, evt)

    def M1_f(self, evt):
        AllCycleKeys(self, evt)

    def M1_Tab(self, evt):
        self.wm.move_focus(focus.MOVE_LEFT)

    def M1_S_Tab(self, evt):
        self.wm.move_focus(focus.MOVE_RIGHT)

    def M1_Return(self, evt):
        MyMoveResizeKeys(self, evt)

    def M1_S_period(self, evt):
        if self.wm.focus_client:
            self.wm.focus_client.delete(1)

    def M1_S_C_period(self, evt):
        if self.wm.focus_client:
            self.wm.focus_client.destroy()

    # View control
    
    def F1(self, evt):
        self.wm.current_screen.view_find_with_client(name('XTerm'))

    def S_F1(self, evt):
        self.wm.system('xterm -geometry 80x25+200+100 &')

    # Pointer  movements
    def M1_Left(self, evt):
        self.wm.display.warp_pointer(-delta.get(evt.time), 0)

    def M1_Right(self, evt):
        self.wm.display.warp_pointer(delta.get(evt.time), 0)

    def M1_Up(self, evt):
        self.wm.display.warp_pointer(0, -delta.get(evt.time))

    def M1_Down(self, evt):
        self.wm.display.warp_pointer(0, delta.get(evt.time))

    # Simulate mouse clicks
    def Any_F9(self, evt):
        self.wm.fake_button_click(1)

    def Any_F10(self, evt):
        self.wm.fake_button_click(2)

    def Any_F11(self, evt):
        self.wm.fake_button_click(3)


class BypassHandler(keys.KeyHandler):
    propagate_keys = 0
    
    def __init__(self, keyhandler):
        keys.KeyHandler.__init__(self, keyhandler.wm)
        self._keyhandler = keyhandler
        self._message = modewindow.Message(.1, modewindow.LEFT, 0, '[Bypassing]')
        self._screen = keyhandler.wm.current_screen
        self._screen.modewindow_add_message(self._message)

    def Pause(self, evt):
        wmanager.debug('keys', 'reinstalling keygrabs')

        self._screen.modewindow_remove_message(self._message)

        # Delete ourself, and reinstall the callee grabs
        self._cleanup()
        self._keyhandler._buildmap()

        # Remove it, just to be sure there are no circular references
        del self._keyhandler
        del self._screen


class MyMoveResizeKeys(MoveResizeKeys):
    h      = MoveResizeKeys._move_w
    l      = MoveResizeKeys._move_e
    k      = MoveResizeKeys._move_n
    j      = MoveResizeKeys._move_s
    
    S_h      = MoveResizeKeys._enlarge_w
    S_l      = MoveResizeKeys._enlarge_e
    S_k      = MoveResizeKeys._enlarge_n
    S_j      = MoveResizeKeys._enlarge_s
    
    C_h      = MoveResizeKeys._shrink_w
    C_l      = MoveResizeKeys._shrink_e
    C_k      = MoveResizeKeys._shrink_n
    C_j      = MoveResizeKeys._shrink_s
    
    Return = MoveResizeKeys._moveresize_end


class AllCycleKeys(CycleKeys):

    Any_h = CycleKeys._cycle_next
    Any_l = CycleKeys._cycle_previous
    
    Any_Escape = CycleKeys._cycle_abort
    Any_Return = CycleKeys._cycle_end


class CycleUMKeys(CycleKeys):
    _cycle_filter = iconified

    Any_h = CycleKeys._cycle_next
    Any_l = CycleKeys._cycle_previous
    
    Any_Escape = CycleKeys._cycle_abort
    Any_Return = CycleKeys._cycle_end


if __name__ == '__main__':
    wmanager.main(ZWM)
